﻿'Created on: 09/12/22
'Author: Jaylin Hochstetler
'A lot of this code was copied from Housekeeper, authored by Robert Mcanny and others. Many thanks to him for making the Housekeeper project open source.
'The purpose of this program is to go through a directory and find any files that don't have a where used and report them to the log file.

Imports RevisionManager
Imports System.Runtime.InteropServices
Imports System.IO
Imports System.Text
Public Class Form1
    Dim LogFilePath As String

    'go through all the listed or selected files, create the log file and run FindUnUsed
    Private Sub ProcessButton_Click(sender As Object, e As EventArgs) Handles ProcessButton.Click
        Dim ErrorMessage As String

        ErrorMessage = CheckStartConditions()
        If ErrorMessage <> "" Then
            Dim result As MsgBoxResult = MsgBox(ErrorMessage, vbOKCancel)
            If result = MsgBoxResult.Cancel Then
                Exit Sub
            End If
            If ErrorMessage.Contains("Please correct the following before continuing") Then
                Exit Sub
            End If
        End If

        CreateLogFile()

        FindUnUsed()

    End Sub
    'open the folder browse dialog and enter the address in the textboxinputdirectory & TextBoxTopLevelFolder & TextBoxLogFile
    Private Sub BrowseButton_Click(sender As Object, e As EventArgs) Handles BrowseButton.Click
        FolderBrowser.ShowDialog()
        TextBoxInputDirectory.Text = FolderBrowser.SelectedPath
        TextBoxTopLevelFolder.Text = FolderBrowser.SelectedPath
        TextBoxLogFile.Text = FolderBrowser.SelectedPath
    End Sub
    'open the folder browse dialog and enter the address in the TextBoxLogFile
    Private Sub ButtonBrwsLog_Click(sender As Object, e As EventArgs) Handles ButtonBrwsLog.Click
        FolderBrowser.ShowDialog()
        TextBoxLogFile.Text = FolderBrowser.SelectedPath
    End Sub
    'open the folder browse dialog and enter the address in the TextBoxTopLevelFolder
    Private Sub ButtonTopLevelFolder_Click(sender As Object, e As EventArgs) Handles ButtonTopLevelFolder.Click
        FolderBrowser.ShowDialog()
        TextBoxTopLevelFolder.Text = FolderBrowser.SelectedPath
    End Sub

    'list all the .par and .psm files in the directory or sub-directories in the listbox and adjust the listbox column based on the MaxFileNameLength
    Private Sub UpdateFileListButton_Click(sender As Object, e As EventArgs) Handles UpdateFileListButton.Click
        If TextBoxInputDirectory.Text <> "" Then
            Dim FoundFiles As IReadOnlyCollection(Of String) = Nothing
            Dim FoundFile As String
            Dim BaseFilename As String
            Dim FileCount As String
            Dim ActiveFileExtensionsList As New List(Of String)

            'this is so if you leave the program open and run it again it will basically reset the status bar
            pBar1.Minimum = 1
            pBar1.Maximum = 1
            pBar1.Visible = False
            TextBoxStatus.Clear()

            ListBoxFiles.Items.Clear()

            'build a list of search extensions based on the checkboxes
            If CheckBoxFilterAsm.Checked Then
                ActiveFileExtensionsList.Add("*.asm")
            End If
            If CheckBoxFilterPar.Checked Then
                ActiveFileExtensionsList.Add("*.par")
            End If
            If CheckBoxFilterPsm.Checked Then
                ActiveFileExtensionsList.Add("*.psm")
            End If

            'notify the user if they uncheck all of the filters
            If ActiveFileExtensionsList.Count = 0 Then
                MsgBox("Please select a file extension.")
                Exit Sub
            End If

            If FileIO.FileSystem.DirectoryExists(TextBoxInputDirectory.Text) Then
                If CheckBoxSearchSubDirectories.Checked Then
                    FoundFiles = FileIO.FileSystem.GetFiles(TextBoxInputDirectory.Text, FileIO.SearchOption.SearchAllSubDirectories, ActiveFileExtensionsList.ToArray)
                Else
                    FoundFiles = FileIO.FileSystem.GetFiles(TextBoxInputDirectory.Text, FileIO.SearchOption.SearchTopLevelOnly, ActiveFileExtensionsList.ToArray)
                End If
            Else
                MsgBox("Input directory does not exist!")
            End If

            Dim MaxFilenameLength As Integer
            MaxFilenameLength = 0
            'go through each FoundFile in FoundFiles, remove the inputdirectory text and add it to the listbox
            If Not FoundFiles Is Nothing Then
                For Each FoundFile In FoundFiles
                    BaseFilename = FoundFile.Replace(TextBoxInputDirectory.Text, "")
                    If Len(BaseFilename) > MaxFilenameLength Then
                        MaxFilenameLength = Len(BaseFilename)
                    End If
                    ListBoxFiles.Items.Add(BaseFilename)
                Next
                ListBoxFiles.ColumnWidth = 12 * MaxFilenameLength
            End If
            'get the file count and display it in label4
            FileCount = GetFileNames.Count
            Label4.Text = FileCount
        End If
    End Sub

    'go through each file and search for a where used from Bottom Up. If there is no where used record it in the log file
    Private Function FindUnUsed() As List(Of String)

        Dim DMApp As DesignManager.Application = Nothing
        Dim FilesToProcess As List(Of String)
        Dim FileToProcess As String
        Dim ObjDoc As DesignManager.Document = Nothing
        Dim TopLevelFolder As String
        Dim AllWhereUsedDocs As New List(Of String)
        Dim WhereUsedDocs As New List(Of DesignManager.Document)
        Dim WhereUsedDoc As DesignManager.Document
        Dim FilesProcessed As Integer
        Dim FileCount As String
        Dim msg As String = ""


        TopLevelFolder = TextBoxTopLevelFolder.Text
        FilesToProcess = GetFileNames()
        FileCount = FilesToProcess.Count


        Try
            DMApp = New DesignManager.Application()
            'make it visible in case of errors
            DMApp.Visible = True
        Catch ex As Exception
        End Try

        pBarSetUp()

        Try

            For Each FileToProcess In FilesToProcess
                Try
                    DMApp.WhereUsedCriteria(Nothing, True) = TopLevelFolder

                    ObjDoc = CType(DMApp.Open(TextBoxInputDirectory.Text & FileToProcess), DesignManager.Document)

                    WhereUsedDoc = CType(
                    DMApp.FindWhereUsed(FileIO.FileSystem.GetFileInfo(TextBoxInputDirectory.Text & FileToProcess)),
                    DesignManager.Document)

                    If WhereUsedDoc Is Nothing Then
                        Try
                            Using writer As New IO.StreamWriter(LogFilePath, True)
                                writer.WriteLine(ObjDoc.FullName)
                            End Using
                        Catch ex As Exception
                            MsgBox("Failed to save log file.")
                        End Try
                    End If

                Catch ex As Exception
                    MsgBox(ex.ToString)
                    GoTo 99
                End Try

                FilesProcessed += 1

                pBar1.PerformStep()

                TextBoxStatus.Text = FilesProcessed.ToString & "/" & FileCount & " files processed."
            Next
99:
        Catch ex As Exception
            MsgBox(ex.ToString)
        Finally
            DMApp.Quit()
        End Try
        msg += "Finished processing " + FilesProcessed.ToString + " files." + Chr(13)
        MsgBox(msg & "See " & LogFilePath & " for details.")
    End Function

    Private Function GetFileNames() As List(Of String)
        ' Build up list of files to process.
        Dim FoundFilesArray As String() = {}
        Dim FoundFilesList As New List(Of String)
        Dim Filename As String
        'if items are selected in the list box
        If ListBoxFiles.SelectedItems.Count > 0 Then
            For i As Integer = 0 To ListBoxFiles.SelectedItems.Count - 1
                Filename = CType(ListBoxFiles.SelectedItems(i), String)
                FoundFilesList.Add(Filename)
            Next

            'if nothing is selected in the list box.
        Else
            For i As Integer = 0 To ListBoxFiles.Items.Count - 1
                Filename = CType(ListBoxFiles.Items(i), String)
                FoundFilesList.Add(Filename)
            Next

        End If

        Return FoundFilesList
    End Function
    Private Sub CreateLogFile()

        Dim Timestamp As String = System.DateTime.Now.ToString("yyyyMMdd_HHmmss")
        LogFilePath = TextBoxLogFile.Text & "\SE_UnusedFileLog_" & Timestamp & ".txt"

        Try
            Using writer As New IO.StreamWriter(LogFilePath, True)
                writer.WriteLine("Files with no where used:")
            End Using
        Catch ex As Exception
            MsgBox("Failed to save log file.")
        End Try
    End Sub
    Private Sub pBarSetUp()
        Dim FilesToProcess As List(Of String)
        FilesToProcess = GetFileNames()

        pBar1.Visible = True

        pBar1.Minimum = 1

        pBar1.Maximum = FilesToProcess.Count

        pBar1.Step = 1
    End Sub

    Private Function CheckStartConditions()
        Dim DMApp As DesignManager.Application = Nothing
        Dim msg As String = ""

        'see if Design Manager is open
        Try
            DMApp = CType(GetObject(, "DesignManager.Application"), DesignManager.Application)
        Catch ex As Exception
        End Try

        If Not DMApp Is Nothing Then
            msg += "    Close Design Manager." + Chr(13)
        End If

        'see if their is anything in the log file box
        If TextBoxLogFile.Text = "" Then
            msg += "    Enter a location for the log file." + Chr(13)
        End If

        'see if there is anything in the top level folder box
        If TextBoxTopLevelFolder.Text = "" Then
            msg += "    Enter a location for the top level folder." + Chr(13)
        End If

        If Len(msg) <> 0 Then
            msg = "Please correct the following before continuing" + Chr(13) + msg
        End If

        Return msg
    End Function

    Private Sub ButtonReadMe_Click(sender As Object, e As EventArgs) Handles ButtonReadMe.Click
        Form2.Show()
    End Sub
End Class